from typing import Optional

import networkx as nx
from duowen_agent.llm import OpenAIChat
from duowen_agent.rag.graph.community_reports import CommunityReportsExtractor
from duowen_agent.rag.graph.merge_graph import MergeGraph
from duowen_agent.rag.graph.prompt import GRAPH_FIELD_SEP
from duowen_agent.rag.graph.query import LocalQuery
from duowen_agent.rag.graph.utils import load_json_graph, dump_graph, get_from_to
from duowen_agent.rag.splitter.base import BaseChunk

from .base import BaseKVStorage, GraphChange, BaseVectorStorage, QueryParam
from .extractor import Extractor
from .query import GlobalQuery
from .storage.kv_json import JsonKVStorage
from .utils import (
    print_call_back,
    is_interrupt,
    tidy_graph,
    compute_args_hash,
)

DEFAULT_ENTITY_TYPES = ["organization", "person", "geo", "event", "category"]


class Graph:
    def __init__(
        self,
        llm_instance: OpenAIChat,
        chunk_func: BaseChunk,
        call_back_func: callable = print_call_back,
        interrupt_func: callable = is_interrupt,
        language: str = "Chinese",
        entity_types: list[str] = None,
        max_gleanings: int = 2,
        extractor_concurrent_num: int = 1,
        community_concurrent_num: int = 1,
        init_graph: nx.Graph = None,
        llm_cache: BaseKVStorage = JsonKVStorage("llm_cache"),
        full_docs: BaseKVStorage = JsonKVStorage("full_docs"),
        text_chunks: BaseKVStorage = JsonKVStorage("text_chunks"),
        knowledge_graph_inst: BaseKVStorage = JsonKVStorage("graph"),
        entity_vdb: BaseVectorStorage = None,
        community_vdb: BaseVectorStorage = None,
        graph_name: str = "duowen_graph",
    ):
        self.llm = llm_instance
        self.language = language
        self.chunk_func = chunk_func
        self.init_graph = init_graph

        # storage
        self.llm_cache = llm_cache
        self.full_docs = full_docs
        self.text_chunks = text_chunks
        self.entity_vdb = entity_vdb
        self.knowledge_graph_inst = knowledge_graph_inst
        self.graph_name = graph_name

        # entity_extractor
        self.entity_types = entity_types or DEFAULT_ENTITY_TYPES
        self.max_gleanings = max_gleanings
        self.extractor_concurrent_num = extractor_concurrent_num

        self.community_concurrent_num = community_concurrent_num
        self.community_report_length = 1500
        self.community_vdb = community_vdb

        # callback interrupt
        self.call_back_func = call_back_func
        self.interrupt_func = interrupt_func

    def get_graph(self) -> Optional[nx.Graph]:
        _data = self.knowledge_graph_inst.get_by_id(self.graph_name)
        if _data:
            return load_json_graph(self.knowledge_graph_inst.get_by_id(self.graph_name))
        else:
            return None

    def _chunk(self, docs: dict[str, str]) -> dict[str, str]:
        _data = {}
        for k, v in docs.items():
            _chunk_data = [i.page_content for i in self.chunk_func.chunk(v)]
            _chunk_id = [f"{k}_{compute_args_hash(i)}" for i in _chunk_data]
            _data.update(dict(zip(_chunk_id, _chunk_data)))

        _data = {k: v for k, v in _data.items() if not self.text_chunks.get_by_id(k)}
        self.text_chunks.upsert(_data)
        return _data

    def insert(self, docs: dict[str, str]):
        for i in [
            self.full_docs,
            self.text_chunks,
            self.entity_vdb,
            self.knowledge_graph_inst,
        ]:
            i.index_start_callback()

        _docs = {k: v for k, v in docs.items() if not self.full_docs.get_by_id(k)}
        if not _docs:
            self.call_back_func("没有需要处理的文件")
            return

        subgraph = self.generate_subgraph(_docs)
        if subgraph:
            self.merge_subgraph(subgraph)

        for i in [
            self.full_docs,
            self.text_chunks,
            self.entity_vdb,
            self.knowledge_graph_inst,
        ]:
            i.index_done_callback()

    def query(self, query: str, param: QueryParam = QueryParam()):

        if param.mode == "local":
            local_query = LocalQuery(
                llm_instance=self.llm,
                community_vdb=self.community_vdb,
                graph=self.get_graph(),
                text_chunks=self.text_chunks,
                entity_vdb=self.entity_vdb,
                call_back_func=self.call_back_func,
                interrupt_func=self.interrupt_func,
                query_param=param,
            )
            return local_query.query(query)

        elif param.mode == "global":
            global_query = GlobalQuery(
                llm_instance=self.llm,
                community_vdb=self.community_vdb,
                call_back_func=self.call_back_func,
                interrupt_func=self.interrupt_func,
                query_param=param,
            )
            return global_query.query(query)

    def build_community(self):
        for i in [self.community_vdb, self.knowledge_graph_inst]:
            i.index_start_callback()

        _graph = self.get_graph()
        for i in _graph.nodes:
            _graph.nodes[i]["clusters"] = []

        cr = CommunityReportsExtractor(
            llm_instance=self.llm,
            llm_cache=self.llm_cache,
            concurrent_num=self.community_concurrent_num,
            call_back_func=self.call_back_func,
            interrupt_func=self.interrupt_func,
            max_report_length=self.community_report_length,
        ).extract(_graph)

        _community_structure = cr.structured_output
        _community_reports = cr.output
        _community_vdata = {
            stru["index"]: (rep, stru)
            for stru, rep in zip(_community_structure, _community_reports)
        }
        self.community_vdb.drop()
        self.community_vdb.upsert(_community_vdata)
        self.knowledge_graph_inst.upsert({self.graph_name: dump_graph(_graph)})

        for i in [self.community_vdb, self.knowledge_graph_inst]:
            i.index_done_callback()

    def generate_subgraph(
        self,
        docs: dict[str, str],
    ) -> Optional[nx.Graph]:
        self.full_docs.upsert(docs)
        chunks = self._chunk(docs)

        if not chunks:
            self.call_back_func("没有要处理的文本块")
            return

        ents, rels = Extractor(
            llm_instance=self.llm,
            language=self.language,
            entity_types=self.entity_types,
            max_gleanings=self.max_gleanings,
            llm_cache=self.llm_cache,
            concurrent_num=self.extractor_concurrent_num,
            call_back_func=self.call_back_func,
            interrupt_func=self.interrupt_func,
        ).extract(chunks)

        subgraph = nx.Graph()
        for ent in ents:
            assert "description" in ent, f"entity {ent} does not have description"
            subgraph.add_node(ent["entity_name"], **ent)

        ignored_rels = 0
        for rel in rels:
            assert "description" in rel, f"relation {rel} does not have description"
            if not subgraph.has_node(rel["src_id"]) or not subgraph.has_node(
                rel["tgt_id"]
            ):
                ignored_rels += 1
                continue
            subgraph.add_edge(
                rel["src_id"],
                rel["tgt_id"],
                **rel,
            )
        if ignored_rels:
            self.call_back_func(f"由于缺少实体而忽略了 {ignored_rels} 关系")
        tidy_graph(subgraph, self.call_back_func, check_attribute=False)
        return subgraph

    def merge_subgraph(
        self,
        subgraph: nx.Graph,
    ):
        MergeGraph(
            graph_name=self.graph_name,
            llm_instance=self.llm,
            knowledge_graph_inst=self.knowledge_graph_inst,
            entity_vdb=self.entity_vdb,
            call_back_func=self.call_back_func,
            interrupt_func=self.interrupt_func,
            concurrent_num=self.extractor_concurrent_num,
            llm_cache=self.llm_cache,
        ).merge(subgraph)



from .bullet import BulletChunker
from .llm import MetaChunker, SectionsChunker
from .markdown import MarkdownHeaderChunker
from .mixin import FastMixinChunker
from .recursive import RecursiveChunker
from .regex import JinaTextChunker
from .semantic import SemanticChunker
from .separator import SeparatorChunker
from .token import TokenChunker
from .word import WordChunker

__all__ = [
    "SectionsChunker",
    "TokenChunker",
    "SeparatorChunker",
    "SemanticChunker",
    "MetaChunker",
    "MarkdownHeaderChunker",
    "RecursiveChunker",
    "FastMixinChunker",
    "WordChunker",
    "JinaTextChunker",
    "BulletChunker",
]



import json
from os import getenv

import httpx
from duowen_agent.mcp.types import ErrorData, JSONRPCError
from duowen_agent.utils.encoders import jsonable_encoder

HTTP_REQUEST_NODE_SSL_VERIFY = (
    getenv("HTTP_REQUEST_NODE_SSL_VERIFY", "true").lower() == "true"
)

SSRF_PROXY_ALL_URL = getenv("SSRF_PROXY_ALL_URL", None)
SSRF_PROXY_HTTP_URL = getenv("SSRF_PROXY_HTTP_URL", None)
SSRF_PROXY_HTTPS_URL = getenv("SSRF_PROXY_HTTPS_URL", None)

SSRF_DEFAULT_TIME_OUT = 5
SSRF_DEFAULT_CONNECT_TIME_OUT = 5
SSRF_DEFAULT_READ_TIME_OUT = 5
SSRF_DEFAULT_WRITE_TIME_OUT = 5


STATUS_FORCELIST = [429, 500, 502, 503, 504]


def create_ssrf_proxy_mcp_http_client(
    headers: dict[str, str] | None = None,
    timeout: httpx.Timeout | None = None,
) -> httpx.Client:
    """Create an HTTPX client with SSRF proxy configuration for MCP connections.

    Args:
        headers: Optional headers to include in the client
        timeout: Optional timeout configuration

    Returns:
        Configured httpx.Client with proxy settings
    """
    if SSRF_PROXY_ALL_URL:
        return httpx.Client(
            verify=HTTP_REQUEST_NODE_SSL_VERIFY,
            headers=headers or {},
            timeout=timeout,
            follow_redirects=True,
            proxy=SSRF_PROXY_ALL_URL,
        )
    elif SSRF_PROXY_HTTP_URL and SSRF_PROXY_HTTPS_URL:
        proxy_mounts = {
            "http://": httpx.HTTPTransport(
                proxy=SSRF_PROXY_HTTP_URL,
                verify=HTTP_REQUEST_NODE_SSL_VERIFY,
            ),
            "https://": httpx.HTTPTransport(
                proxy=SSRF_PROXY_HTTPS_URL,
                verify=HTTP_REQUEST_NODE_SSL_VERIFY,
            ),
        }
        return httpx.Client(
            verify=HTTP_REQUEST_NODE_SSL_VERIFY,
            headers=headers or {},
            timeout=timeout,
            follow_redirects=True,
            mounts=proxy_mounts,
        )
    else:
        return httpx.Client(
            verify=HTTP_REQUEST_NODE_SSL_VERIFY,
            headers=headers or {},
            timeout=timeout,
            follow_redirects=True,
        )


def ssrf_proxy_sse_connect(url, **kwargs):
    """Connect to SSE endpoint with SSRF proxy protection.

    This function creates an SSE connection using the configured proxy settings
    to prevent SSRF attacks when connecting to external endpoints.

    Args:
        url: The SSE endpoint URL
        **kwargs: Additional arguments passed to the SSE connection

    Returns:
        EventSource object for SSE streaming
    """
    from httpx_sse import connect_sse

    # Extract client if provided, otherwise create one
    client = kwargs.pop("client", None)
    if client is None:
        # Create client with SSRF proxy configuration
        timeout = kwargs.pop(
            "timeout",
            httpx.Timeout(
                timeout=SSRF_DEFAULT_TIME_OUT,
                connect=SSRF_DEFAULT_CONNECT_TIME_OUT,
                read=SSRF_DEFAULT_READ_TIME_OUT,
                write=SSRF_DEFAULT_WRITE_TIME_OUT,
            ),
        )
        headers = kwargs.pop("headers", {})
        client = create_ssrf_proxy_mcp_http_client(headers=headers, timeout=timeout)
        client_provided = False
    else:
        client_provided = True

    # Extract method if provided, default to GET
    method = kwargs.pop("method", "GET")

    try:
        return connect_sse(client, method, url, **kwargs)
    except Exception:
        # If we created the client, we need to clean it up on error
        if not client_provided:
            client.close()
        raise


def create_mcp_error_response(
    request_id: int | str | None, code: int, message: str, data=None
):
    """Create MCP error response"""
    error_data = ErrorData(code=code, message=message, data=data)
    json_response = JSONRPCError(
        jsonrpc="2.0",
        id=request_id or 1,
        error=error_data,
    )
    json_data = json.dumps(jsonable_encoder(json_response))
    sse_content = f"event: message\ndata: {json_data}\n\n".encode()
    yield sse_content



import json
import os
from typing import List, Dict, Any, TypedDict

from duowen_agent.llm import OpenAIChat, OpenAIEmbedding
from duowen_agent.llm.tokenizer import tokenizer
from duowen_agent.rag.nlp import LexSynth

from .base import BaseMemory
from ...utils.core_utils import stream_to_string


class ConversationMessage(TypedDict):
    role: str
    content: str
    content_split: str
    content_vector: list[float]


class ConversationMemory(BaseMemory):
    """
    Memory system for storing and summarizing conversation history.
    """

    def __init__(
        self,
        llm: OpenAIChat,
        lex_synth: LexSynth,
        emb: OpenAIEmbedding,
        summarize_threshold: int = 4000,
        path: str = None,
    ):
        self.llm = llm
        self.lex_synth = lex_synth
        self.emb = emb
        self.summarize_threshold = summarize_threshold
        self.messages: List[ConversationMessage] = []
        self.summary = ""
        self.summary_split = ""
        self.last_summarized = 0  # Index of last summarized message
        self.path = path

    def _dump(self) -> Dict[str, Any]:
        return {
            "messages": self.messages,
            "summary": self.summary,
            "summary_split": self.summary_split,
            "last_summarized": self.last_summarized,
        }

    def _load(self, data: Dict[str, Any]) -> None:
        self.messages = data["messages"]
        self.summary = data["summary"]
        self.summary_split = data["summary_split"]
        self.last_summarized = data["last_summarized"]

    def save_to_disk(self) -> None:
        with open(self.path, "w", encoding="utf-8") as f:
            f.write(json.dumps(self._dump(), ensure_ascii=False))

    def load_from_disk(
        self,
    ) -> None:
        if self.path and os.path.exists(self.path):
            with open(self.path, "r", encoding="utf-8") as f:
                data = json.load(f)
            self._load(data)

    def _add_message(self, role: str, message: str) -> None:
        self.messages.append(
            ConversationMessage(
                role=role,
                content=message,
                content_split=self.lex_synth.content_cut(message),
                content_vector=self.emb.get_embedding(
                    tokenizer.truncate_emb(message, self.emb.max_token)
                )[0],
            )
        )

    def add_user_message(self, message: str) -> None:
        self._add_message("user", message)
        self._check_and_summarize()

    def add_assistant_message(self, message: str) -> None:
        self._add_message("assistant", message)
        self._check_and_summarize()

    def add_system_message(self, message: str) -> None:
        for msg in self.messages:
            if msg["role"] == "system" and msg["content"] == message:
                return  # Skip if identical system message exists

        # Insert at beginning to maintain system message priority
        self.messages.insert(0, {"role": "system", "content": message})

    def get_messages(self, token_limit: int = 10240) -> List[Dict[str, str]]:
        _messages = [
            {"role": m["role"], "content": m["content"]} for m in self.messages
        ]
        if self.summary:
            system_messages = [
                {"role": "system", "content": m["content"]}
                for m in _messages
                if m["role"] == "system"
            ]
            token_limit -= tokenizer.messages_len(system_messages)

            summary_message = {
                "role": "system",
                "content": f"Conversation summary: {self.summary}",
            }
            token_limit -= tokenizer.messages_len([summary_message])

            last_message = _messages[self.last_summarized :]
            token_limit -= tokenizer.messages_len(last_message)

            truncated_other = []
            for i in reversed(
                [m for m in _messages if m["role"] != "system"][: self.last_summarized]
            ):
                if tokenizer.messages_len([i]) <= token_limit:
                    truncated_other.insert(0, i)
                    token_limit -= tokenizer.messages_len([i])
                else:
                    break
            return system_messages + [summary_message] + truncated_other + last_message
        else:
            system_messages = [
                {"role": "system", "content": m["content"]}
                for m in _messages
                if m["role"] == "system"
            ]
            token_limit -= tokenizer.messages_len(system_messages)
            truncated_other = []
            for i in reversed([m for m in self.messages if m["role"] != "system"]):
                if tokenizer.messages_len([i]) <= token_limit:
                    truncated_other.insert(0, i)
                    token_limit -= tokenizer.messages_len([i])
                else:
                    break
            return system_messages + truncated_other

    def get_chat_history(self) -> str:
        """
        Get a formatted string of the chat history.

        Returns:
            Formatted chat history
        """
        history = []

        # Add summary if available
        if self.summary:
            history.append(f"Summary: {self.summary}\n")

        # Add messages (excluding system messages)
        for msg in self.messages:
            if msg["role"] != "system":
                history.append(f"{msg['role'].capitalize()}: {msg['content']}")

        return "\n".join(history)

    def clear(self) -> None:
        """Clear all messages from memory except system messages."""
        system_messages = [msg for msg in self.messages if msg["role"] == "system"]
        self.messages = system_messages
        self.summary = ""
        self.summary_split = ""
        self.last_summarized = 0

    def search(
        self,
        query: str,
        tkweight: float = 0.7,
        vtweight: float = 0.3,
        threshold: float = 0.5,
    ) -> List[Dict[str, Any]]:

        results = []
        msg = [msg for msg in self.messages if msg["role"] != "system"]

        _relevance_score = self.lex_synth.hybrid_similarity(
            question=query,
            question_vector=self.emb.get_embedding(query)[0],
            docs_vector=[i["content_vector"] for i in msg],
            docs_sm=[i["content_split"] for i in msg],
            tkweight=tkweight,
            vtweight=vtweight,
            qa=True,
        )

        for index, relevance in enumerate(_relevance_score):
            results.append(
                {
                    "role": msg[index]["role"],
                    "content": msg[index]["content"],
                    "relevance": relevance,
                }
            )

        if self.summary and self.summary_split:

            results.insert(
                0,
                {
                    "content": f"From summary: {self.summary}",
                    "role": "system",
                    "relevance": self.lex_synth.query_text_similarity(
                        question=query, docs_sm=[self.summary_split]
                    )[0],
                },
            )

        return [i for i in results if i["relevance"] > threshold]

    def _check_and_summarize(self) -> None:
        """Check if summarization is needed and summarize if necessary."""
        # Count tokens in non-system messages
        non_system_messages = [
            {"role": m["role"], "content": m["content"]}
            for m in self.messages
            if m["role"] != "system"
        ]
        token_count = tokenizer.messages_len(non_system_messages)
        # print(f"token_count: {token_count}")
        if token_count > self.summarize_threshold:
            self._summarize_conversation()

    def _summarize_conversation(self) -> None:
        """Summarize the conversation and remove older messages."""
        # Get non-system messages for summarization
        non_system_messages = [m for m in self.messages if m["role"] != "system"]

        # Keep the most recent messages
        keep_count = min(10, len(non_system_messages) // 3)
        keep_count = max(keep_count, 3)  # 保留最近3轮的不进行摘要

        # Messages to summarize (only those not previously summarized)
        to_summarize = (
            non_system_messages[self.last_summarized : -keep_count]
            if keep_count > 0
            else non_system_messages[self.last_summarized :]
        )

        if not to_summarize:
            return

        # Format conversation for summarization
        conversation = "\n".join(
            [f"{msg['role'].capitalize()}: {msg['content']}" for msg in to_summarize]
        )

        # Create summarization prompt
        prompt = f"""
对下列对话进行总结，提取最重要的信息、关键事实以及达成的任何决定或结论。保持未来对话所需的基本上下文。

历史摘要: {self.summary if self.summary else "None"}

需总结的对话内容:
{conversation}

请提供一个全面、详尽的摘要，涵盖所有关键信息。
"""
        # print(prompt)
        # Generate summary
        new_summary = stream_to_string(
            self.llm.chat_for_stream(messages=prompt, max_new_tokens=800)
        )

        # Update summary
        if self.summary:
            self.summary = f"{self.summary}\n\nUpdated with: {new_summary}"
        else:
            self.summary = new_summary

        self.summary_split = self.lex_synth.content_cut(self.summary)

        self.last_summarized = len(non_system_messages) - keep_count



import json
import traceback
from abc import ABC, abstractmethod
from hashlib import sha1
from typing import List, Dict, Literal, Annotated
from urllib.parse import urlparse
from urllib.request import urlopen

import requests
from pydantic import BaseModel, conlist, Field

from duowen_agent.error import EmbeddingError
from duowen_agent.utils.cache import Cache


class EmbeddingVLInputModel(BaseModel):
    input: conlist(
        Annotated[Dict[Literal["text", "image"], str], Field(min_length=1)],
        min_length=1,
    )


class EmbeddingVL(ABC):
    def __init__(
            self,
            base_url: str,
            model: str = None,
            api_key: str = None,
            timeout: int = 120,
            dimension: int = 512,
            extra_headers: dict = None,
            **kwargs,
    ):
        self.base_url = f"{base_url}/v1/embeddings"
        self.model = model
        self.timeout = timeout
        self.api_key = api_key
        self.dimension = dimension
        self.extra_headers = extra_headers
        self.kwargs = kwargs

    @abstractmethod
    def get_embedding(
            self, input: List[Dict[Literal["text", "image"], str]]
    ) -> List[List[float]]:
        pass


class JinaClipV2Embedding(EmbeddingVL):
    def __init__(
            self,
            base_url: str,
            model: str = None,
            api_key: str = None,
            timeout: int = 120,
            dimension: int = 512,
            extra_headers: dict = None,
            **kwargs,
    ):
        super().__init__(
            base_url, model, api_key, timeout, dimension, extra_headers, **kwargs
        )
        self.model = model or kwargs.get("model_name", "jina-clip-v2")

    def get_embedding(
            self, input: List[Dict[Literal["text", "image"], str]]
    ) -> List[List[float]]:
        try:
            EmbeddingVLInputModel(input=input)

            headers = {"Content-Type": "application/json"}
            if self.extra_headers:
                headers.update(self.extra_headers)
            if self.api_key:
                headers["Authorization"] = f"Bearer {self.api_key}"

            data = {
                "model": self.model,
                "dimensions": self.dimension,
                "normalized": self.kwargs.get("normalized", True),
                "embedding_type": self.kwargs.get("embedding_type", "float"),
                "input": input,
            }
            if (embedding_type := self.kwargs.get("task", None)) == "retrieval.query":
                data["task"] = embedding_type

            resp = requests.post(
                self.base_url,
                headers=headers,
                data=json.dumps(data),
                timeout=self.timeout,
            )
            resp.raise_for_status()
            v_res = resp.json()

            return [item["embedding"] for item in v_res["data"]]
        except:
            raise EmbeddingError(traceback.format_exc(), self.base_url, self.api_key)


class EmbeddingVLCache:
    def __init__(
            self, emb_cache: Cache, model_instance: EmbeddingVL, cache_ttl: int = 3600
    ):
        self.emb_cache = emb_cache
        self.model_instance = model_instance
        self.cache_ttl = cache_ttl
        self.base_url = self.model_instance.base_url
        self.model = self.model_instance.model
        self.timeout = self.model_instance.timeout
        self.api_key = self.model_instance.api_key
        self.dimension = self.model_instance.dimension

    def _generate_key(self, question: str, is_image: bool = False) -> str:
        if not is_image:
            return f"embed::{self.model_instance.model}::{sha1(question.encode()).hexdigest()}"
        else:
            resp = urlopen(urlparse(question)._replace(query=None).geturl())
            if resp.getcode() == 200:
                return f"embed::{self.model_instance.model}::{sha1(resp.read()).hexdigest()}"
            else:
                raise EmbeddingError(
                    traceback.format_exc(), self.base_url, self.api_key
                )

    def get_embedding(
            self, input: List[Dict[Literal["text", "image"], str]], cache_ttl: int = None
    ) -> List[List[float]]:

        EmbeddingVLInputModel(input=input)
        _cache_ttl = cache_ttl or self.cache_ttl or 3600

        embedding_cache = {
            embedding_content: {
                "key": self._generate_key(embedding_content, embedding_type == "image"),
                "embedding_type": embedding_type,
                "embedding": [],
            }
            for item in input
            for embedding_type, embedding_content in item.items()
        }
        cached_embeddings = self.emb_cache.mget(
            [embedding_cache[item]["key"] for item in embedding_cache]
        )
        for i, item in enumerate(embedding_cache):
            if cached_embeddings[i]:
                embedding_cache[item]["embedding"] = json.loads(cached_embeddings[i])

        _input = [
            {embedding_cache[embedding_content]["embedding_type"]: embedding_content}
            for embedding_content in embedding_cache
            if not embedding_cache[embedding_content]["embedding"]
        ]
        cached_embeddings = self.model_instance.get_embedding(_input)
        for i, item in enumerate(embedding_cache):
            if not embedding_cache[item]["embedding"]:
                embedding_cache[item]["embedding"] = cached_embeddings[i]
                self.emb_cache.set(
                    embedding_cache[item]["key"],
                    json.dumps(cached_embeddings[i]),
                    _cache_ttl,
                )

        return [embedding_cache[item]["embedding"] for item in embedding_cache]



from abc import ABC, abstractmethod

from .utils import fullwidth_to_halfwidth, traditional_to_simplified


class BaseTokenizer(ABC):

    @abstractmethod
    def freq(self, tk):
        pass

    @abstractmethod
    def tag(self, tk):
        pass

    @abstractmethod
    def tokenize(self, line):
        pass

    @abstractmethod
    def fine_grained_tokenize(self, tks):
        pass

    @abstractmethod
    def add_tok(self, token, freq, tag):
        pass

    @staticmethod
    def _strQ2B(line):
        return fullwidth_to_halfwidth(line)

    strQ2B = _strQ2B

    @staticmethod
    def _tradi2simp(line):
        return traditional_to_simplified(line)

    tradi2simp = _tradi2simp



import math
import re
from typing import List, Literal, Optional

from duowen_agent.rag.models import Document

from .base import BaseChunk

BULLET_PATTERN = {
    # 英文法律条款（PART/Chapter/Section等）
    "english_legal": [
        r"\*{0,2}?PART (ONE|TWO|THREE|FOUR|FIVE|SIX|SEVEN|EIGHT|NINE|TEN)",
        r"\*{0,2}?Chapter (I+V?|VI*|XI|IX|X)",
        r"\*{0,2}?Section [0-9]+",
        r"\*{0,2}?Article [0-9]+",
    ],
    # 中文法律条款结构（编/章/节/条 + 括号条款）
    "chinese_legal": [
        r"^\*{0,2}?第[零一二三四五六七八九十百0-9]+(分?编|部分)",
        r"^\*{0,2}?第[零一二三四五六七八九十百0-9]+篇",
        r"^\*{0,2}?第[零一二三四五六七八九十百0-9]+章",
        r"^\*{0,2}?第[零一二三四五六七八九十百0-9]+节",
        r"^\*{0,2}?第[零一二三四五六七八九十百0-9]+条",
        r"^\*{0,2}?第[零一二三四五六七八九十百0-9]+款",
        r"^\*{0,2}?第[零一二三四五六七八九十百0-9]+项",
        r"^\*{0,2}?[$（][零一二三四五六七八九十百]+[$）]",
        r"^\*{0,2}?[零一二三四五六七八九十百]+[ 、]",
        # r"^\*{0,2}?[0-9]{1,2}[\. 、]",
        # r"^\*{0,2}?[0-9]{1,2}\.[0-9]{1,2}[^a-zA-Z/%~-]",
        # r"^\*{0,2}?[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{1,2}",
        # r"^\*{0,2}?[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{1,2}",
        # r"^\*{0,2}?[$（][0-9]{1,2}[$）]",
    ],
    "official_document": [
        # 1. 顶层结构（较少见，多用于大型公文）
        r"^\*{0,2}?第[零一二三四五六七八九十百0-9]+编",  # 如“第一编 总则”
        # 2. 章（公文核心层级）
        r"^\*{0,2}?第[零一二三四五六七八九十百0-9]+章",  # 如“第一章 总体要求”
        # 3. 节（章下细分）
        r"^\*{0,2}?第[零一二三四五六七八九十百0-9]+节",  # 如“第一节 主要目标”
        # 4. 条（节下基础单位）
        r"^\*{0,2}?第[零一二三四五六七八九十百0-9]+条",  # 如“第一条 指导思想”
        # 5. 款（条下细分，通常无编号，若明确标注则包含）
        r"^\*{0,2}?第[零一二三四五六七八九十百0-9]+款",  # 如“第一款 适用范围”
        # 6. 项（款下细分，公文常用中文序号）
        r"^\*{0,2}?第[零一二三四五六七八九十百0-9]+项",  # 如“第一项 具体措施”
        r"^\*{0,2}?[一二三四五六七八九十百]+[、．]",  # 如“一、”“二．”（中文一级序号）
        r"^\*{0,2}?[（(][一二三四五六七八九十百]+[）)]",  # 如“（一）”“(二)”（中文二级序号）
        # 7. 目（项的进一步细分，常用阿拉伯数字）
        r"^\*{0,2}?[0-9]+[、．]",  # 如“1、”“2．”（阿拉伯数字一级）
        r"^\*{0,2}?[（(][0-9]+[）)]",  # 如“（1）”“(2)”（阿拉伯数字二级）
        r"^\*{0,2}?[①②③④⑤⑥⑦⑧⑨⑩]",  # 如“①”“②”（圈码序号）
    ],
}


def not_bullet(line):
    patt = [r"\*{0,2}?0", r"\*{0,2}?[0-9]+ +[0-9~个只-]", r"\*{0,2}?[0-9]+\.{2,}"]
    return any([re.match(r, line) for r in patt])


def is_bullet_document(text):
    """
    判断文档是否属于条款类结构
    :param text: 输入文本
    :param threshold: 匹配比例阈值（默认5%）
    :return: True/False 及匹配详情
    """
    total_lines = len(text.split("\n"))

    # 动态阈值规则（基于行数的简单分段）
    if total_lines <= 5:
        min_lines = 2  # 5行以内至少2行匹配
    elif total_lines <= 20:
        min_lines = max(2, math.ceil(total_lines * 0.3))  # 20行以内30%
    else:
        min_lines = max(3, math.ceil(total_lines * 0.05))  # 长文档5%

    def _structured(bullet_type, text):
        match_count = 0

        # 合并所有正则模式
        all_patterns = [re.compile(p) for p in BULLET_PATTERN[bullet_type]]

        # 逐行检测
        for line in text.split("\n"):
            stripped_line = "".join(filter(str.isprintable, line)).strip()
            if not stripped_line:
                continue

            # 判断是否匹配条款模式
            for pattern in all_patterns:
                if pattern.match(stripped_line) and not not_bullet(stripped_line):
                    match_count += 1
                    break  # 避免重复计数

        return {
            "bullet_type": bullet_type,
            "matched_lines": match_count,
            "total_lines": total_lines,
        }

    data_list = []
    for i in BULLET_PATTERN:
        # 暂时不处理法律文档识别（法律和公文行文相似）
        if i == "chinese_legal":
            continue
        data_list.append(_structured(i, text))

    data = {}
    for x in data_list:
        if x["matched_lines"] > min_lines:
            data = x
            break

    if not data:
        data = max(data_list, key=lambda x: x["matched_lines"])

    data["is_structured"] = data["matched_lines"] > min_lines

    return data


def split_bullet_text(text: str, pattern: str) -> List[str]:
    """
    Markdown目录分割
    输入: 内容；目录级别
    输出: List[[分割后的内容(不包含当前标题), 当前级别标题]]
    - 自动包含第一个匹配标题前的所有内容
    - 正确处理代码块中的伪标题
    """
    lines = text.split("\n")
    result = []
    current_content = []

    re_pattern = re.compile(pattern)

    for line in lines:
        stripped_for_check = "".join(filter(str.isprintable, line.strip()))

        if re_pattern.match(stripped_for_check):
            result.append("\n".join(current_content))
            current_content = [line]
            continue
        else:
            current_content.append(line)

    if current_content:
        result.append("\n".join(current_content))

    return result


class BulletChunker(BaseChunk):
    def __init__(
        self,
        chunk_size: int = 512,
        token_count_type: Literal["o200k", "cl100k"] = "cl100k",
        bullet_pattern_type: Optional[
            Literal["english_legal", "chinese_legal", "official_document"]
        ] = None,
        **kwargs
    ):
        super().__init__(token_count_type=token_count_type)
        self.chunk_size = chunk_size
        self.bullet_pattern_type = bullet_pattern_type

    def chunk(self, text: str) -> List[Document]:

        if self.bullet_pattern_type is None:
            _chk = is_bullet_document(text)

            if _chk["is_structured"] is False:
                return [
                    Document(
                        page_content=text,
                        metadata=dict(
                            token_count=self.token_len(text),
                            chunk_index=0,
                            is_structured=False,
                        ),
                    )
                ]
            rule_pattern = BULLET_PATTERN[_chk["bullet_type"]]
        else:
            rule_pattern = BULLET_PATTERN[self.bullet_pattern_type]

        _doc = []

        def _chunk(text, level):
            _data = split_bullet_text(text, rule_pattern[level])

            if len(_data) == 1:
                if (level + 1) < len(rule_pattern):
                    _chunk(text, level + 1)
                else:
                    _doc.append(text)
                return

            _current_doc = []

            for i in _data:

                if self.token_len(i) > self.chunk_size:

                    if _current_doc:
                        _doc.append("\n\n".join(_current_doc))
                        _current_doc = []

                    if (level + 1) < len(rule_pattern):

                        _chunk(i, level + 1)
                    else:
                        _doc.append(i)
                else:
                    _merge_str = "\n\n".join(_current_doc + [i])
                    if self.token_len(_merge_str) <= self.chunk_size:
                        _current_doc.append(i)
                    else:
                        _doc.append("\n\n".join(_current_doc))
                        _current_doc = [i]

            if _current_doc:
                _doc.append("\n\n".join(_current_doc))
                _current_doc = []

        _chunk(text, 0)

        return [
            Document(
                page_content=i,
                metadata=dict(token_count=self.token_len(i), chunk_index=idx),
            )
            for idx, i in enumerate(_doc)
            if len(i.strip()) > 0
        ]



import os
from abc import ABC, abstractmethod
from typing import List, Optional, Union, Literal

import json5
from langchain_openai import ChatOpenAI
from openai import OpenAI, AsyncOpenAI
from openai.types.chat import ChatCompletionToolParam

from duowen_agent.error import LLMError, LengthLimitExceededError, MaxTokenExceededError
from duowen_agent.llm.entity import (
    MessagesSet,
    Message,
    openai_params_list,
)
from duowen_agent.llm.utils import format_messages
from duowen_agent.tools.base import BaseTool
from duowen_agent.utils.core_utils import (
    record_time,
    async_record_time,
)
from .entity import ToolsCall, Tool


class BaseAIChat(ABC):
    base_url: str
    api_key: Optional[str]
    model: str
    token_limit: int

    @abstractmethod
    def chat_for_stream(
        self,
        messages: str | List[dict] | List[Message] | MessagesSet,
        **kwargs,
    ):
        raise NotImplementedError

    @abstractmethod
    def chat(
        self,
        messages: str | List[dict] | List[Message] | MessagesSet,
        **kwargs,
    ):
        raise NotImplementedError

    async def achat(
        self,
        messages: str | List[dict] | List[Message] | MessagesSet,
        **kwargs,
    ):
        raise NotImplementedError

    async def achat_for_stream(
        self,
        messages: str | List[dict] | List[Message] | MessagesSet,
        **kwargs,
    ):
        raise NotImplementedError


class OpenAIChat(BaseAIChat):
    LANGCHAIN_CHAT_PARAMS_LIST = [
        "model",
        "api_key",
        "base_url",
        "openai_api_base",  # base_url的别名
        "timeout",
        "max_retries",
        "organization",
        "openai_organization",  # organization的别名
        "openai_proxy",
        "default_headers",
        "default_query",
        "temperature",
        "top_p",
        "max_tokens",
        "extra_body",
        "n",
        "seed",
        "frequency_penalty",
        "presence_penalty",
        "logprobs",
        "top_logprobs",
        "logit_bias",
        "stop",
        "stream",
        "streaming",  # stream的别名
        "functions",
        "tools",
        "tool_choice",
        "response_format",
        "service_tier",
        "user",
        # 用户标识
        "http_client",  # HTTP客户端
        "http_async_client",  # 异步HTTP客户端
        "reasoning_effort",  # 推理努力程度
        "reasoning",  # 推理相关
        "store",  # 存储选项
        "truncation",  # 截断选项
    ]

    def __init__(
        self,
        base_url: str = None,
        model: str = None,
        api_key: str = None,
        temperature: float = 0.2,
        top_p: float = None,
        presence_penalty: float = None,
        frequency_penalty: float = None,
        timeout: int = 120,
        token_limit: int = 32 * 1024,
        is_reasoning: bool = False,
        extra_headers: dict = None,
        extra_body: dict = None,
        max_retries: int = 2,
        **kwargs,
    ):
        """
           temperature 控制生成文本的随机性（值越低，输出越确定）。
           top_p 通过概率阈值控制输出的多样性（与 temperature 二选一）。
           presence_penalty 对已出现的内容进行惩罚（避免重复话题）。
           frequency_penalty 对高频内容进行惩罚（避免重复用词）。

           精确模式： 0.1, 0.3, 0.4, 0.7
           平衡模式： 0.5, 0.5, 0.4, 0.7
           自由模式： 0.9, 0.9, 0.4, 0.2

        qwen3 xinference 关闭推理
        "extra_body": {"chat_template_kwargs": {"enable_thinking": False}},

        qwen3 硅基流动 关闭推理
        "extra_body": {"enable_thinking": False},


        """
        self.base_url = base_url or os.environ.get("OPENAI_BASE_URL", None)
        self.api_key = api_key or os.environ.get("OPENAI_API_KEY", "xxx")
        self.temperature = temperature
        self.top_p = top_p
        self.presence_penalty = presence_penalty
        self.frequency_penalty = frequency_penalty
        self.model = model or kwargs.get("model_name", None) or "gpt-3.5-turbo"
        self.timeout = timeout
        self.token_limit = token_limit
        self.is_reasoning = is_reasoning
        self.extra_headers = extra_headers
        self.extra_body = extra_body
        self.max_retries = max_retries
        self.kwargs = kwargs

        # qwen3 开启推理模式
        if self.is_reasoning and not self.extra_body and "qwen3" in self.model.lower():
            self.extra_body = {"enable_thinking": True}

    @property
    def sync_client(self) -> OpenAI:
        return OpenAI(
            api_key=self.api_key, base_url=self.base_url, timeout=self.timeout
        )

    @property
    def async_client(self) -> AsyncOpenAI:  # 新增异步客户端属性
        return AsyncOpenAI(
            api_key=self.api_key, base_url=self.base_url, timeout=self.timeout
        )

    @property
    def langchain_client(self) -> ChatOpenAI:

        # 过滤参数，只传递 ChatOpenAI 支持的参数
        filtered_kwargs = {
            k: v for k, v in self.kwargs.items() if k in self.LANGCHAIN_CHAT_PARAMS_LIST
        }

        if self.model and "model" not in filtered_kwargs:
            filtered_kwargs["model"] = self.model
        if self.api_key and "api_key" not in filtered_kwargs:
            filtered_kwargs["api_key"] = self.api_key
        if self.base_url and "base_url" not in filtered_kwargs:
            filtered_kwargs["base_url"] = self.base_url

        if self.extra_headers:
            filtered_kwargs["default_headers"] = self.extra_headers

        if self.extra_body:
            filtered_kwargs["extra_body"] = self.extra_body

        if self.timeout:
            filtered_kwargs["timeout"] = self.timeout

        if self.temperature:
            filtered_kwargs["temperature"] = self.temperature

        if self.top_p:
            filtered_kwargs["top_p"] = self.top_p

        if self.presence_penalty:
            filtered_kwargs["presence_penalty"] = self.presence_penalty

        if self.frequency_penalty:
            filtered_kwargs["frequency_penalty"] = self.frequency_penalty

        if self.max_retries:
            filtered_kwargs["max_retries"] = self.max_retries

        return ChatOpenAI(**filtered_kwargs)

    def _check_message(
        self, message: str | List[dict] | List[Message] | MessagesSet
    ) -> MessagesSet:
        return format_messages(message, self.is_reasoning)

    def _build_params(
        self,
        messages: str | List[dict] | List[Message] | MessagesSet,
        tools: List[Union[BaseTool, ChatCompletionToolParam]] = None,
        tool_choice: Optional[
            Union[dict, str, Literal["auto", "none", "required", "any"], bool]
        ] = "auto",
        temperature: float = None,
        top_p: float = None,
        presence_penalty: float = None,
        frequency_penalty: float = None,
        max_tokens: int = None,
        timeout: int = 30,
        forced_reasoning: bool = False,
        **kwargs,
    ):
        _message = self._check_message(messages)
        if self.is_reasoning:
            _message.remove_assistant_think()

        if self.is_reasoning and forced_reasoning and _message[-1].role == "user":
            _message.add_assistant("<think>\n")

        _params = {"messages": _message.get_messages(), "model": self.model}

        if tools:
            _tools = []
            for tool in tools:
                if isinstance(tool, BaseTool):
                    _tools.append({"type": "function", "function": tool.to_schema()})
                elif isinstance(tool, dict):
                    _tools.append(tool)
                else:
                    raise ValueError(f"Unsupported tool type: {type(tool)}")

            _params["tools"] = _tools
            if tool_choice:
                _params["tool_choice"] = tool_choice

        if temperature:
            _params["temperature"] = temperature
        elif self.temperature:
            _params["temperature"] = self.temperature

        if top_p:
            _params["top_p"] = top_p
        elif self.top_p:
            _params["top_p"] = self.top_p

        if presence_penalty:
            _params["presence_penalty"] = presence_penalty
        elif self.presence_penalty:
            _params["presence_penalty"] = self.presence_penalty

        if frequency_penalty:
            _params["frequency_penalty"] = frequency_penalty
        elif self.frequency_penalty:
            _params["frequency_penalty"] = self.frequency_penalty

        if max_tokens:
            _params["max_tokens"] = max_tokens

        if timeout:
            _params["timeout"] = timeout
        elif self.timeout:
            _params["timeout"] = self.timeout

        if self.extra_headers:
            _params["extra_headers"] = self.extra_headers

        if self.extra_body:
            _params["extra_body"] = self.extra_body

        if self.kwargs:
            for k, v in self.kwargs.items():
                if k in openai_params_list and k not in _params:
                    _params[k] = v

        if kwargs:
            for k, v in kwargs.items():
                if k in openai_params_list and k not in _params:
                    _params[k] = v
        return _params

    @record_time()
    def chat_for_stream(
        self,
        messages: str | List[dict] | List[Message] | MessagesSet,
        tools: List[Union[BaseTool, ChatCompletionToolParam]] = None,
        tool_choice: Optional[
            Union[dict, str, Literal["auto", "none", "required", "any"], bool]
        ] = "auto",
        temperature: float = None,
        top_p: float = None,
        presence_penalty: float = None,
        frequency_penalty: float = None,
        max_tokens: int = None,
        timeout: int = 30,
        forced_reasoning=False,
        **kwargs,
    ):
        if tools:
            raise NotImplementedError("stream模式 不支持使用工具调用")

        _params = self._build_params(
            messages=messages,
            tools=tools,
            tool_choice=tool_choice,
            temperature=temperature,
            top_p=top_p,
            presence_penalty=presence_penalty,
            frequency_penalty=frequency_penalty,
            max_tokens=max_tokens,
            timeout=timeout,
            forced_reasoning=forced_reasoning,
            **kwargs,
        )
        _params["stream"] = True

        try:

            response = self.sync_client.chat.completions.create(**_params)

            _full_message = ""
            _is_think_start = False
            _is_think_end = False

            for chunk in response:

                if chunk.choices:
                    _content_msg = chunk.choices[0].delta.content or ""
                    _reasoning_content_msg = (
                        chunk.choices[0].delta.reasoning_content or ""
                        if hasattr(chunk.choices[0].delta, "reasoning_content")
                        else ""
                    )

                    if _reasoning_content_msg and _is_think_start is False:
                        _msg = f"<think>\n{_reasoning_content_msg}"
                        _is_think_start = True
                    elif (
                        _content_msg
                        and _is_think_start is True
                        and _is_think_end is False
                    ):
                        _msg = f"\n</think>\n\n{_content_msg}"
                        _is_think_end = True
                    elif _reasoning_content_msg and _is_think_end is False:
                        _msg = _reasoning_content_msg
                    else:
                        _msg = _content_msg

                    _full_message += _msg

                    if _msg:
                        yield _msg

                    if chunk.choices[0].finish_reason == "length":
                        raise LengthLimitExceededError(content=_full_message)
                    elif chunk.choices[0].finish_reason == "max_tokens":
                        raise MaxTokenExceededError(content=_full_message)

            if not _full_message:  # 如果流式输出返回为空
                raise LLMError(
                    "语言模型流式输出无响应", self.base_url, self.model, messages
                )

        except (LengthLimitExceededError, MaxTokenExceededError) as e:
            raise e

        except Exception as e:
            raise LLMError(str(e), self.base_url, self.model, messages)

    async def achat_for_stream(
        self,
        messages: str | List[dict] | List[Message] | MessagesSet,
        tools: List[Union[BaseTool, ChatCompletionToolParam]] = None,
        tool_choice: Optional[
            Union[dict, str, Literal["auto", "none", "required", "any"], bool]
        ] = "auto",
        temperature: float = None,
        top_p: float = None,
        presence_penalty: float = None,
        frequency_penalty: float = None,
        max_tokens: int = None,
        timeout: int = 30,
        forced_reasoning=False,
        **kwargs,
    ):
        if tools:
            raise NotImplementedError(
                "stream模式 不支持使用工具调用，使用 chat_for_stream 时请不要传入 tools 参数"
            )

        _params = self._build_params(
            messages=messages,
            tools=tools,
            tool_choice=tool_choice,
            temperature=temperature,
            top_p=top_p,
            presence_penalty=presence_penalty,
            frequency_penalty=frequency_penalty,
            max_tokens=max_tokens,
            timeout=timeout,
            forced_reasoning=forced_reasoning,
            **kwargs,
        )
        _params["stream"] = True

        try:

            response = await self.async_client.chat.completions.create(**_params)

            _full_message = ""
            _is_think_start = False
            _is_think_end = False

            async for chunk in response:
                if chunk.choices:
                    _content_msg = chunk.choices[0].delta.content or ""
                    _reasoning_content_msg = (
                        chunk.choices[0].delta.reasoning_content or ""
                        if hasattr(chunk.choices[0].delta, "reasoning_content")
                        else ""
                    )

                    if _reasoning_content_msg and _is_think_start is False:
                        _msg = f"<think>\n{_reasoning_content_msg}"
                        _is_think_start = True
                    elif (
                        _content_msg
                        and _is_think_start is True
                        and _is_think_end is False
                    ):
                        _msg = f"\n</think>\n\n{_content_msg}"
                        _is_think_end = True
                    elif _reasoning_content_msg and _is_think_end is False:
                        _msg = _reasoning_content_msg
                    else:
                        _msg = _content_msg

                    if _msg:
                        _full_message += _msg
                        yield _msg

                    if chunk.choices[0].finish_reason == "length":
                        yield ""
                        raise LengthLimitExceededError(content=_full_message)
                    elif chunk.choices[0].finish_reason == "max_tokens":
                        yield ""
                        raise MaxTokenExceededError(content=_full_message)

            if not _full_message:  # 如果流式输出返回为空
                raise LLMError(
                    "语言模型流式输出无响应", self.base_url, self.model, messages
                )

        except (LengthLimitExceededError, MaxTokenExceededError) as e:
            raise e

        except Exception as e:
            raise LLMError(str(e), self.base_url, self.model, messages)

    @record_time()
    def chat(
        self,
        messages: str | List[dict] | List[Message] | MessagesSet,
        tools: List[Union[BaseTool, ChatCompletionToolParam]] = None,
        tool_choice: Optional[
            Union[dict, str, Literal["auto", "none", "required", "any"], bool]
        ] = "auto",
        temperature: float = None,
        top_p: float = None,
        presence_penalty: float = None,
        frequency_penalty: float = None,
        max_tokens: int = None,
        timeout: int = 30,
        forced_reasoning: bool = False,
        **kwargs,
    ):

        _params = self._build_params(
            messages=messages,
            tools=tools,
            tool_choice=tool_choice,
            temperature=temperature,
            top_p=top_p,
            presence_penalty=presence_penalty,
            frequency_penalty=frequency_penalty,
            max_tokens=max_tokens,
            timeout=timeout,
            forced_reasoning=forced_reasoning,
            **kwargs,
        )
        _params["stream"] = False

        try:
            response = self.sync_client.chat.completions.create(**_params)
            if response.choices[0].finish_reason == "tool_calls":
                _reasoning_content_msg = (
                    response.choices[0].message.reasoning_content
                    if hasattr(response.choices[0].message, "reasoning_content")
                    else ""
                )
                _content_msg = response.choices[0].message.content

                _tool_calls = response.choices[0].message.tool_calls

                return ToolsCall(
                    think=f"<think>\n{_reasoning_content_msg}</think>\n\n{_content_msg}",
                    tools=[
                        Tool(
                            name=i.function.name,
                            arguments=json5.loads(i.function.arguments),
                        )
                        for i in _tool_calls
                    ],
                )

            elif response.choices[0].finish_reason == "length":
                raise LengthLimitExceededError(
                    content=response.choices[0].message.content
                )
            elif response.choices[0].finish_reason == "max_tokens":
                raise MaxTokenExceededError(content=response.choices[0].message.content)
            else:
                _reasoning_content_msg = (
                    response.choices[0].message.reasoning_content
                    if hasattr(response.choices[0].message, "reasoning_content")
                    else ""
                )
                _content_msg = response.choices[0].message.content

                if _content_msg:
                    if _reasoning_content_msg:
                        return f"<think>\n{_reasoning_content_msg}</think>\n\n{_content_msg}"
                    return _content_msg
                else:
                    raise LLMError(
                        "语言模型无消息回复", self.base_url, self.model, messages
                    )

        except (LengthLimitExceededError, MaxTokenExceededError) as e:
            raise e

        except Exception as e:
            raise LLMError(str(e), self.base_url, self.model, messages)

    @async_record_time()
    async def achat(
        self,
        messages: str | List[dict] | List[Message] | MessagesSet,
        tools: List[Union[BaseTool, ChatCompletionToolParam]] = None,
        tool_choice: Optional[
            Union[dict, str, Literal["auto", "none", "required", "any"], bool]
        ] = "auto",
        temperature: float = None,
        top_p: float = None,
        presence_penalty: float = None,
        frequency_penalty: float = None,
        max_tokens: int = None,
        timeout: int = 30,
        forced_reasoning: bool = False,
        **kwargs,
    ):

        _params = self._build_params(
            messages=messages,
            tools=tools,
            tool_choice=tool_choice,
            temperature=temperature,
            top_p=top_p,
            presence_penalty=presence_penalty,
            frequency_penalty=frequency_penalty,
            max_tokens=max_tokens,
            timeout=timeout,
            forced_reasoning=forced_reasoning,
            **kwargs,
        )
        _params["stream"] = False

        try:
            response = await self.async_client.chat.completions.create(**_params)
            if response.choices[0].finish_reason == "tool_calls":

                _reasoning_content_msg = (
                    response.choices[0].message.reasoning_content
                    if hasattr(response.choices[0].message, "reasoning_content")
                    else ""
                )
                _content_msg = response.choices[0].message.content

                _tool_calls = response.choices[0].message.tool_calls
                _tool_call = _tool_calls[0]
                _tool_name = _tool_call.function.name

                return ToolsCall(
                    think=f"<think>\n{_reasoning_content_msg}</think>\n\n{_content_msg}",
                    tools=[
                        Tool(
                            name=i.function.name,
                            arguments=json5.loads(i.function.arguments),
                        )
                        for i in _tool_calls
                    ],
                )

            elif response.choices[0].finish_reason == "length":
                raise LengthLimitExceededError(
                    content=response.choices[0].message.content
                )
            elif response.choices[0].finish_reason == "max_tokens":
                raise MaxTokenExceededError(content=response.choices[0].message.content)
            else:
                _reasoning_content_msg = (
                    response.choices[0].message.reasoning_content
                    if hasattr(response.choices[0].message, "reasoning_content")
                    else ""
                )
                _content_msg = response.choices[0].message.content

                if _content_msg:
                    if _reasoning_content_msg:
                        return f"<think>\n{_reasoning_content_msg}</think>\n\n{_content_msg}"
                    return _content_msg
                else:
                    raise LLMError(
                        "语言模型无消息回复", self.base_url, self.model, messages
                    )

        except Exception as e:
            raise LLMError(str(e), self.base_url, self.model, messages)
